library flutter_easybus;

import 'dart:async';
typedef void  EventCallback<T>(T arg); ///构造注册回调
var removeKeySet = List<int>.from({}); /// 异步删除防止在迭代器中操作元素
class EventSubscription<T>{
  Map<int,dynamic> map;
  late int _key;
  Completer<T> ?completer;
  EventCallback ?_hookCall;
  EventSubscription({required this.map});
  /// 监听事件回调
  EventSubscription<T> listen(EventCallback<T> callback){
    this._key=callback.hashCode;
    this.map[_key]=callback;
    return this;
  }
  _waitCall(T t) {
    if(!completer!.isCompleted){
      completer!.complete(t);
    }else{
      free();
    }
  }
  /// 使用fluttery异步实现 ,注意！！！！回调一次之后会自动销毁
  Future<T> wait() async{
    completer = new Completer();
    _hookCall= (t)=>_waitCall(t);
    _key=_hookCall.hashCode;
    map[_key]=_hookCall!;
    return completer!.future;
  }
  free(){
    try{
      if(_hookCall==null){
        map.remove(_key);
      }
    }catch(e){
      // print(e)
    }finally{
      removeKeySet.add(_key);
      completer=null;
      _hookCall=null;
    }
  }
}
///* 功能：创建EasyBus工具类
class EasyBus {
  static bool get isRelease => bool.fromEnvironment("dart.vm.product");
  /// 私有构造函数
  EasyBus._internal();

  /// 保存单例
  static EasyBus _singleton = new EasyBus._internal();
  static Type typeOf<T>() => T;
  /// 工厂构造函数
  factory EasyBus() => _singleton;

  var _mainMap = new Map<Type,Map<int,dynamic>>();

  ///注册事件
  EventSubscription<T> on <T>(){
    _mainMap[typeOf<T>()] ??= new Map<int,dynamic>();
    return new EventSubscription<T>(map:_mainMap[typeOf<T>()] as  Map<int,dynamic> );
  }
  EventSubscription<SimpleBean> simpleOn (){
    _mainMap[typeOf<SimpleBean>()] ??= new Map<int,dynamic>();
    return new EventSubscription<SimpleBean>(map:_mainMap[typeOf<SimpleBean>()] as  Map<int,dynamic> );
  }
  void simpleSend(String key,dynamic obj) {
    send(SimpleBean(key: key, obj: obj));
  }


  ///事件发送
  void send(dynamic data) {
    if( _mainMap.containsKey(data.runtimeType)){
      var mapItem= _mainMap[data.runtimeType];
      if(mapItem!.isEmpty){
        _mainMap.remove(data.runtimeType);///清理无效注册
        return;
      }
      mapItem.forEach((key, value) {
        value(data);
      });
      removeKeySet.forEach((element) {
        mapItem.remove(element);
      });
      removeKeySet.clear();
    }
  }
  void clear(){
    _mainMap.clear();
  }
  ///释放回调
  // ignore: unnecessary_question_mark
  void free<T>({dynamic ? data}){
    Type key=typeOf<int>();
    if(data!=null){
      key=data.runtimeType;
    }else{
      key= typeOf<T>();
    }
    var item= _mainMap[key];
    if(item !=null){
      if(data==null){
        item.clear();
        item=null;
        _mainMap.remove(key);
      }else{
        item.remove(data.hashCode);
      }
    }

  }

}
class SimpleBean{
  late String key;
  late dynamic  obj;
  SimpleBean({required this.key,required this.obj});
}
